function [Gred,Q,Epodrel,Efitrel]=IODMD(X,U,Y,dt,Nq)
% function [Gred,Q,Epod,Efit]=IODMD(X,U,Y,dt,Nq)
% OR
% function [Gred,Q,Epod,Efit]=IODMD(X,U,Y,dt,Q)
%
% This function implements the IODMD method for constructing a reduced
% order, discrete-time system.  The reduced order system is constructed
% from snapshots of the nonlinear system with a forced response.
%
% Inputs:
%  *X = Nx-by-Nt matrix of state snapshots. The i^th column X(:,i) is a
%        snapshot of the state at the i^th time.
%  *U = Nu-by-Nt matrix of input snapshots.
%  *Y = Ny-by-Nt matrix of output snapshots.
%  *dt = Sample time in sec.  It is assumed that the snapshots are
%     recorded every dt seconds.
%  *Nq = Integer specifying the number of modes to select for the model.
%        Nq can also be specified as a vector of integers. In this case
%        models are constructed for each dimension specified in Nq.
%  *Q = Alternative syntax where Q is specified as an Nx-by-Nq matrix
%    of modes. The i^th column Q(:,i) is the i^th mode for the model.
%
% Outputs:
%  *Gred = Discrete-time reduced order model with Nq states.
%  *Q = Nx-by-Nq matrix of modes used to construct the reduced order model.
%    [This output Q is the same as the input Q in the alternative syntax.]
%  *Epod = Relative error for POD reduction.
%    [Epod is set to zero in alternative syntax where Q is an input.]
%  *Efit = Relative error for model (least-squares) fit.
%
% Note: If Nq is a vector of numNq integers then Gred is a numNq-by-1
% array of reduced order models and Epod/Efit are numNq-by-1 vectors of
% relative errors. The i^th entry of Gred, Epod, Efit correspond to the
% reduced-order model constructed of dimension Nq(i). Q is returned as an
% Nx-by-Nq(end)  matrix of modes.  The modes used to construct the
% i^th model are given by Q(:,1:Nq(i)).

% Stack Snapshots
X0 = X(:,1:end-1);
X1 = X(:,2:end);
Y0 = Y(:,1:end-1);
U0 = U(:,1:end-1);

% Determine syntax case
if isvector(Nq) && all( floor(Nq)==ceil(Nq) )
    % Input is vector of integers specifying model dimensions
    numNq = numel(Nq);
    
    % SVD of state snapshots
    [UU,SS,~]=svd(X0);
    SSd = diag(SS);
    
    % Flag to specify syntax where Nq is a vector of integers
    flg = true;
else
    % Input corresponds to user-specified modes
    Q = Nq;
    Nq = size(Q,2);
    numNq = 1;
    
    % Flag to specify syntax where Q specifies modes to use
    flg = false;
end

% Compute reduced order model for specified state dimensions
Ny = size(Y,1);
Nu = size(U,1);
Gred = ss( zeros(Ny,Nu,numNq) );
Epodrel = zeros(numNq,1);
Efitrel = zeros(numNq,1);
for i=1:numNq
    % Number of modes
    Nqi = Nq(i);
    
    if flg        
        % Select modes
        Q = UU(:,1:Nqi);
        
        % Relative energy lost in POD modes
        Epodrel(i) = sum(SSd(Nqi+1:end).^2)/sum(SSd.^2);
    end
    
    % Best reduced order fit of chosen dimension 
    QX1 = Q'*X1;
    QX0 = Q'*X0;
    FGHD = [QX1; Y0]/[QX0; U0];
    
    % Extract state matrices and form reduced order model
    F = FGHD(1:Nqi,1:Nqi);
    G = FGHD(1:Nqi,Nqi+1:end);
    H = FGHD(Nqi+1:end,1:Nqi);
    D = FGHD(Nqi+1:end,Nqi+1:end);
    Gred(:,:,i) = ss(F,G,H,D,dt);
    
    % Relative model fit error
    if nargout==4
        % To save computation, only compute error if requested by user
        if i==1
            normXY = norm([X1;Y0],'fro');
        end
        Efit = [X1;Y0]-blkdiag(Q,1)*FGHD*[QX0;U0];
        Efitrel(i) = norm(Efit,'fro')/normXY;
    end
end

